import static java.lang.Math.*;


public class atomFunc{
    static vectorFunc v;
    
    public atomFunc()
    {
        v = new vectorFunc();
    }
    
    public static atom zeroforce(atom i)    // reset force accumulator
    {
        atom j = new atom(i);
        j.force = new vector(0,0);    
        return j;
    }
    
    public static atom restingTest(atom i)
    {
        atom j = new atom(i);
        if (i.vel.x >= -50 && i.vel.x <= 50 && i.vel.y >= -50 && i.vel.y <= 50)
        {
            j.rested = true;
            return j;
        }
        else
        {
            j.rested = false;
            return j;
        }
    }
    
    public static boolean getrest(atom i)
    {        
        return i.rested;
    }
    
    public static atom unrest(atom i)
    {
        atom j = new atom(i);
        j.rested = false;
        return j;
    }
    
    public static atom rested(atom i)
    {
        atom j = new atom(i);
        j.rested = true;
        return j;
    }
    
    public static float distanceCenters(atom i, atom j)    // find distance between atoms (center to center)
    {
        return v.mag(v.sub(i.pos, j.pos));
    }
    
    public static float distanceCentersSQ(atom i, atom j)    // find distance between atoms (center to center)
    {
        return v.magSQ(v.sub(i.pos, j.pos));
    }
    
    public static float distanceSurfaces(atom i, atom j)    // find distance between atoms (from surface to surface)
    {
        return v.mag(v.sub(i.pos, j.pos)) - sizes(i, j);
    }
    
    public static float distanceSurfacesSQ(atom i, atom j)    // find distance between atoms (from surface to surface)
    {
        return v.magSQ(v.sub(i.pos, j.pos)) - sizes(i, j);
    }
    
    public static vector separate(atom i)        // return randomized separation force adjusted for mass
    {
        float rx = (float)Math.random()+0.5f;
        float ry = (float)Math.random()-0.5f;
        return v.scale(new vector(rx, ry), rx+ry*1000000*i.mass);
    }
    
    public static atom addforce(atom i, vector f)    // accumulate force
    {
        atom j = new atom(i);
        j.force = v.add(j.force, f);
        return j;
    }
    
    public static atom subforce(atom i, vector f)    // accumulate force
    {
        atom j = new atom(i);
        j.force = v.sub(j.force, f);
        return j;
    }
    
    public static float sizes(atom i, atom j)    // return combined radius of two atoms
    {
        return i.radius+j.radius;
    }
                                           
    public static vector repulse(atom i, atom j, float restlen, float distcent)    // return self repulsion spring+damper forces
    {
        vector link = v.sub(i.pos, j.pos);
        double mult = restlen / (float)sqrt(distcent);
        float comp = (i.mass*j.mass)/(i.mass+j.mass);
        
        vector spring = v.scale(v.sub(v.scale(link, mult), link), comp*220000); // dist=distance surface to surface. (how deep) K=5000
        vector damper = v.scale((v.sub(j.vel, i.vel)), 10);            // D=damping
        return v.add(spring, damper);
    }
    
    public static atom gravity(atom i, vector grav)        // apply gravity on atom
    {
        atom j = new atom(i);
        j.force = v.add(j.force, v.scale(grav, j.mass));
        return j;
    }
    
    public static atom integrate(atom i, float dt)        // integrate system
    {
        atom j = new atom(i);
        j.vel = v.add(j.vel, v.scale(v.invscale(j.force, j.mass), dt));    // v = (f/m) *dt   or   v = a*dt  or  v = integrated v
        j.pos = v.add(j.pos, v.scale(j.vel, dt));                    // x += vel *dt
        
        return j;
    }
    
    public static atom hardcollision(atom i, vector boxSize, vector boxPos, float decay, float friction) // only for 0' and 90' angles
    {
        atom j = new atom(i);
        
        if (j.pos.x-j.radius < boxPos.x)     //else if can miss corner collisions
        {
            j.pos = new vector(boxPos.x+j.radius, j.pos.y);
            j.vel = new vector(j.vel.x * -decay, j.vel.y * friction);    
        }
        if (j.pos.x+j.radius > boxPos.x+boxSize.x)
        {
            j.pos = new vector(boxPos.x+boxSize.x-j.radius, j.pos.y);
            j.vel = new vector(j.vel.x * -decay, j.vel.y * friction);
        }
        if (j.pos.y-j.radius < boxPos.y)
        {
            j.pos = new vector(j.pos.x, boxPos.y+j.radius);
            j.vel = new vector(j.vel.x * friction, j.vel.y * -decay);
        }
        if (j.pos.y+j.radius > boxPos.y+boxSize.y)
        {
            j.pos = new vector(j.pos.x, boxPos.y+boxSize.y-j.radius);
            j.vel = new vector(j.vel.x * friction, j.vel.y * -decay);
        }
        return j;
    }
    
    public static atom terminalforces(atom i)        // apply gravity on atom
    {
        atom j = new atom(i);
        if (j.force.x > 8000000)
        {
            j.force = new vector(8000000,j.force.y);
        }
        if (j.force.x < -8000000)
        {
            j.force = new vector(-8000000,j.force.y);
        }
        if (j.force.y > 8000000)
        {
            j.force = new vector(j.force.x,8000000);
        }
        if (j.force.y < -8000000)
        {
            j.force = new vector(j.force.x,-8000000);
        }
        return j;
    }
    
    public static atom terminalvelocity(atom i)        // apply gravity on atom
    {
        atom j = new atom(i);
        if (j.vel.x > 10000)
        {
            j.vel = new vector(10000,j.vel.y);
        }
        if (j.vel.x < -10000)
        {
            j.vel = new vector(-10000,j.vel.y);
        }
        if (j.vel.y > 10000)
        {
            j.vel = new vector(j.vel.x,10000);
        }
        if (j.vel.y < -10000)
        {
            j.vel = new vector(j.vel.x,-10000);
        }
        return j;
    }
    
    public static atom velocitydrag(atom i)        // apply gravity on atom
    {
        atom j = new atom(i);
        j.vel = v.scale(j.vel, 0.998f);
        return j;
    }
    
    public static float getmass(atom i)    // reset force accumulator
    {       
        return i.mass;
    }
    
    public static vector getpos(atom i)    // get pos
    {       
        return i.pos;
    }
    
    public static vector getvel(atom i)    // get vel
    {       
        return i.vel;
    }
    
    public static float getsize(atom i)    // get size
    {       
        return i.radius;
    }
};
